﻿namespace TabsStudioCommon
{
    class SolutionEvents
    {
        public delegate void EventDelegate();
        public event EventDelegate OnAfterCloseSolution;
        public event EventDelegate OnAfterOpenSolution;
        public event EventDelegate OnBeforeCloseSolution;

        public SolutionEvents(EnvDTE80.DTE2 dte)
        {
            solutionEventsSink = new Sink(this);
            System.IServiceProvider serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(
                dte as Microsoft.VisualStudio.OLE.Interop.IServiceProvider);
            vsSolution = serviceProvider.GetService(typeof(Microsoft.VisualStudio.Shell.Interop.SVsSolution)) as
                Microsoft.VisualStudio.Shell.Interop.IVsSolution;
            vsSolution.AdviseSolutionEvents(solutionEventsSink, out sinkCookie);
        }

        public void Close()
        {
            vsSolution.UnadviseSolutionEvents(sinkCookie);
        }

        private Sink solutionEventsSink;
        private Microsoft.VisualStudio.Shell.Interop.IVsSolution vsSolution;
        private uint sinkCookie;

        private class Sink : Microsoft.VisualStudio.Shell.Interop.IVsSolutionEvents
        {
            public Sink(SolutionEvents events)
            {
                this.events = events;
            }

            public int OnAfterCloseSolution(System.Object pUnkReserved)
            {
                try
                {
                    if (events.OnAfterCloseSolution != null)
                        events.OnAfterCloseSolution.Invoke();
                }
                catch (System.Exception ex)
                {
                    HandleException(ex);
                }
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnAfterLoadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pStubHierarchy,
                Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnAfterOpenProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
                int fAdded)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnAfterOpenSolution(System.Object pUnkReserved, int fNewSolution)
            {
                try
                {
                    if (events.OnAfterOpenSolution != null)
                        events.OnAfterOpenSolution.Invoke();
                }
                catch (System.Exception ex)
                {
                    HandleException(ex);
                }
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnBeforeCloseProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
                int fRemoved)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnBeforeCloseSolution(System.Object pUnkReserved)
            {
                try
                {
                    if (events.OnBeforeCloseSolution != null)
                        events.OnBeforeCloseSolution.Invoke();
                }
                catch (System.Exception ex)
                {
                    HandleException(ex);
                }
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnBeforeUnloadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy,
                Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pStubHierarchy)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnQueryCloseProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
                int fRemoving, ref int pfCancel)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnQueryCloseSolution(System.Object pUnkReserved, ref int pfCancel)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            public int OnQueryUnloadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy,
                ref int pfCancel)
            {
                return Microsoft.VisualStudio.VSConstants.S_OK;
            }

            private void HandleException(System.Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message + System.Environment.NewLine + ex.StackTrace,
                    "Tabs Studio (Unhandled exception)",
                    System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Error);
            }

            private SolutionEvents events;
        }
    }
}
