﻿namespace TabsStudioOrderKeeper
{
    public class Controller
    {
        public Controller(IOpenDocuments openDocuments)
        {
            this.openDocuments = openDocuments;
            model = new Model();
            orderAnalyser = new OrderAnalyser(model, openDocuments);
        }

        public void SaveOrder(TabsStudioExt.TabExtensionEventArgs e)
        {
            TabOrder tabOrder = orderAnalyser.GetAroundOrder(GetTabList(e.Tabs), e.Tab, e.Tab);
            tabOrder.tab = openDocuments.FindDocument(e.TabExtension);
            model.SetTabOrder(tabOrder);
        }

        public void RestoreOrder(TabsStudioExt.TabExtensionEventArgs e)
        {
            TabOrder savedOrder = model.GetTabOrder(openDocuments.FindDocument(e.TabExtension));
            if (savedOrder == null)
                return;

            TabsStudioExt.Tab bestTabPosition = orderAnalyser.FindBestPosition(e.Tab, savedOrder, GetTabList(e.Tabs));
            if (bestTabPosition != null && bestTabPosition.TabItem != e.Tab.TabItem)
            {
                e.Tabs.TabPanel2.Children.Remove(e.Tab.TabItem);
                e.Tabs.TabPanel2.Children.Insert(e.Tabs.TabPanel2.Children.IndexOf(bestTabPosition.TabItem), e.Tab.TabItem);
            }
        }

        public void ResetOrder()
        {
            model.Reset();
        }

        private System.Collections.Generic.List<TabsStudioExt.Tab> GetTabList(TabsStudioExt.Tabs tabs)
        {
            System.Collections.Generic.List<TabsStudioExt.Tab> tabList =
                new System.Collections.Generic.List<TabsStudioExt.Tab>(tabs.TabList);
            tabList.Sort(
                delegate (TabsStudioExt.Tab lhs, TabsStudioExt.Tab rhs)
                {
                    return tabs.TabPanel2.Children.IndexOf(lhs.TabItem).CompareTo(
                                tabs.TabPanel2.Children.IndexOf(rhs.TabItem));
                });
            return tabList;
        }

        private IOpenDocuments openDocuments;
        private Model model;
        private OrderAnalyser orderAnalyser;
    }
}
