﻿namespace TabsStudioOrderKeeper
{
    class OrderAnalyser
    {
        public OrderAnalyser(Model model, IOpenDocuments openDocuments)
        {
            this.model = model;
            this.openDocuments = openDocuments;
        }

        public TabOrder GetAroundOrder(System.Collections.Generic.List<TabsStudioExt.Tab> tabList,
            TabsStudioExt.Tab center, TabsStudioExt.Tab exclude)
        {
            TabOrder tabOrder = new TabOrder();
            bool before = true;
            foreach (TabsStudioExt.Tab i in tabList)
            {
                if (i.TabItem == center.TabItem)
                    before = false;
                if (exclude != null && i.TabItem == exclude.TabItem)
                    continue;
                (before ? tabOrder.before : tabOrder.after).AddRange(openDocuments.FindDocuments(i));
            }
            return tabOrder;
        }

        public TabsStudioExt.Tab FindBestPosition(TabsStudioExt.Tab tab, TabOrder savedOrder,
            System.Collections.Generic.List<TabsStudioExt.Tab> tabList)
        {
            int bestScore = 0;
            System.Collections.Generic.List<TabsStudioExt.Tab> bestTabPositions =
                new System.Collections.Generic.List<TabsStudioExt.Tab>();
            for (int i = 0; i < tabList.Count; ++i)
            {
                TabOrder candidateOrder = GetAroundOrder(tabList, tabList[i], tab);
                int candidateScore = GetScore(candidateOrder, savedOrder);
                if (candidateScore > bestScore)
                {
                    bestScore = candidateScore;
                    bestTabPositions.Clear();
                    bestTabPositions.Add(tabList[i]);
                }
                else
                {
                    if (candidateScore == bestScore)
                        bestTabPositions.Add(tabList[i]);
                }
            }
            if (bestTabPositions.Count == 0)
                return null;

            return GetRelativeBest(bestTabPositions, tabList, tab);
        }

        private int GetScore(TabOrder candidateOrder, TabOrder savedOrder)
        {
            int result = 0;
            foreach (string c in candidateOrder.before)
            {
                foreach (string s in savedOrder.before)
                {
                    if (string.Compare(c, s, true) == 0)
                    {
                        ++result;
                        break;
                    }
                }
            }
            foreach (string c in candidateOrder.after)
            {
                foreach (string s in savedOrder.after)
                {
                    if (string.Compare(c, s, true) == 0)
                    {
                        ++result;
                        break;
                    }
                }
            }
            return result;
        }

        private TabsStudioExt.Tab GetRelativeBest(System.Collections.Generic.List<TabsStudioExt.Tab> candidatePositions,
            System.Collections.Generic.List<TabsStudioExt.Tab> tabList, TabsStudioExt.Tab tab)
        {
            if (candidatePositions.Count == 1)
                return candidatePositions[0];

            foreach (TabsStudioExt.Tab p in candidatePositions)
            {
                if (!AnyOrderViolations(p, candidatePositions[0], candidatePositions[candidatePositions.Count - 1],
                    tabList, tab))
                    return p;
            }
            return candidatePositions[0];
        }

        private bool AnyOrderViolations(TabsStudioExt.Tab position, TabsStudioExt.Tab first, TabsStudioExt.Tab last,
            System.Collections.Generic.List<TabsStudioExt.Tab> tabList, TabsStudioExt.Tab tab)
        {
            int positionIndex = TabIndex(position, tabList);
            int firstIndex = TabIndex(first, tabList);
            int lastIndex = TabIndex(last, tabList);
            int tabIndex = TabIndex(tab, tabList);
            string tabID = openDocuments.FindDocument(tab.TabExtensions[0]);
            for (int i = firstIndex; i < lastIndex; ++i)
            {
                if (i == tabIndex)
                    continue;

                foreach (TabsStudioExt.TabExtension e in tabList[i].TabExtensions)
                {
                    TabOrder helperOrder = model.GetTabOrder(openDocuments.FindDocument(e));
                    if (helperOrder != null)
                    {
                        if (positionIndex <= i && AfterContains(helperOrder, tabID))
                            return true;
                        if (positionIndex > i && BeforeContains(helperOrder, tabID))
                            return true;
                    }
                }
            }
            return false;
        }

        private bool AfterContains(TabOrder order, string id)
        {
            foreach (string s in order.after)
            {
                if (string.Compare(id, s, true) == 0)
                    return true;
            }
            return false;
        }

        private bool BeforeContains(TabOrder order, string id)
        {
            foreach (string s in order.before)
            {
                if (string.Compare(id, s, true) == 0)
                    return true;
            }
            return false;
        }

        private int TabIndex(TabsStudioExt.Tab tab, System.Collections.Generic.List<TabsStudioExt.Tab> tabList)
        {
            for (int i = 0; i < tabList.Count; ++i)
            {
                if (tabList[i].TabItem == tab.TabItem)
                    return i;
            }
            return -1;
        }

        private Model model;
        private IOpenDocuments openDocuments;
    }
}
