﻿namespace TabsStudioOrderKeeper
{
    public class OrderKeeper : TabsStudioExt.ITabsStudioAddin
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine, EnvDTE80.DTE2 dte)
        {
            this.engine = engine;
            openDocuments = new TabsStudioCommon.OpenDocuments();
            openDocumentsAdapter = new OpenDocumentsAdapter(openDocuments);
            controller = new Controller(openDocumentsAdapter);
            solutionEvents = new TabsStudioCommon.SolutionEvents(dte);
            superGroupProperties = new TabsStudioExt.Util.SuperGroupProperties();

            engine.TabExtensionCreated += OnTabExtensionCreated;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;
            engine.TabExtensionRenamed += OnTabExtensionRenamed;

            solutionEvents.OnAfterCloseSolution += OnAfterCloseSolution;
        }

        public void OnDisconnection()
        {
            engine.TabExtensionCreated -= OnTabExtensionCreated;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            engine.TabExtensionRenamed -= OnTabExtensionRenamed;

            solutionEvents.OnAfterCloseSolution -= OnAfterCloseSolution;

            solutionEvents.Close();
        }

        private void OnTabExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            openDocuments.OnTabExtensionCreated(e);
            if (e.Tab.TabExtensions.Count == 1)
            {
                if (superGroupProperties.GetName(e.Tab.TabItem) != null)
                    return;
                controller.RestoreOrder(e);
            }
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            controller.SaveOrder(e);
            openDocuments.OnTabExtensionDestroyed(e);
        }

        private void OnTabExtensionRenamed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            openDocuments.OnTabExtensionRenamed(e);
        }

        private void OnAfterCloseSolution()
        {
            controller.ResetOrder();
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private TabsStudioCommon.OpenDocuments openDocuments;
        private OpenDocumentsAdapter openDocumentsAdapter;
        private Controller controller;
        private TabsStudioCommon.SolutionEvents solutionEvents;
        private TabsStudioExt.Util.SuperGroupProperties superGroupProperties;
    }
}