﻿using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace UnitTestProject
{
    [TestClass]
    public class UnitTestController
    {
        [TestMethod]
        public void TestOpen1()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("1", tabs, controller);
            Assert.AreEqual(1, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(0), "Title");
        }

        [TestMethod]
        public void TestRestore1()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("1", tabs, controller);
            Open("2", tabs, controller);
            Close(0, tabs, controller);
            Open("1", tabs, controller);

            Assert.AreEqual(2, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(0), "Title");
        }

        [TestMethod]
        public void TestRestore2()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("1", tabs, controller);
            Open("2", tabs, controller);
            Open("3", tabs, controller);
            Close(1, tabs, controller);
            Open("2", tabs, controller);

            Assert.AreEqual(3, tabs.GetCount(), "Count");
            Assert.AreEqual("2", tabs.GetTitle(1), "Title");
        }

        [TestMethod]
        public void TestRestoreTwice()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("0", tabs, controller);
            Open("1", tabs, controller);
            Open("2", tabs, controller);
            Open("3", tabs, controller);

            Close(1, tabs, controller);
            Open("1", tabs, controller);
            Assert.AreEqual(4, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(1), "Title");

            tabs.Move(1, 2);
            Close(2, tabs, controller);
            Open("1", tabs, controller);
            Assert.AreEqual(4, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(2), "Title");
        }

        [TestMethod]
        public void TestClose123Restore31()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("1", tabs, controller);
            Open("2", tabs, controller);
            Open("3", tabs, controller);
            Close(0, tabs, controller);
            Close(0, tabs, controller);
            Close(0, tabs, controller);
            Open("3", tabs, controller);
            Open("1", tabs, controller);

            Assert.AreEqual(2, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(0), "Title");
        }

        [TestMethod]
        public void TestClose321Restore31()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("1", tabs, controller);
            Open("2", tabs, controller);
            Open("3", tabs, controller);
            Close(2, tabs, controller);
            Close(1, tabs, controller);
            Close(0, tabs, controller);
            Open("3", tabs, controller);
            Open("1", tabs, controller);

            Assert.AreEqual(2, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(0), "Title");
        }

        [TestMethod]
        public void TestClose13Restore21()
        {
            OpenDocumentsMock docs = new OpenDocumentsMock();
            Tabs tabs = new Tabs(docs);
            TabsStudioOrderKeeper.Controller controller = new TabsStudioOrderKeeper.Controller(docs);

            Open("1", tabs, controller);
            Open("3", tabs, controller);
            Close(0, tabs, controller);
            Close(0, tabs, controller);
            Open("2", tabs, controller);
            Open("1", tabs, controller);

            Assert.AreEqual(2, tabs.GetCount(), "Count");
            Assert.AreEqual("1", tabs.GetTitle(0), "Title");
        }

        private void Open(string title, Tabs tabs, TabsStudioOrderKeeper.Controller controller)
        {
            tabs.Open(title);
            controller.RestoreOrder(tabs.GetExtensionArgs(tabs.GetCount() - 1, 0));
        }

        private void Close(int index, Tabs tabs, TabsStudioOrderKeeper.Controller controller)
        {
            controller.SaveOrder(tabs.GetExtensionArgs(index, 0));
            tabs.Close(index);
        }
    }
}
